-- This file is a part of Avoision.
-- 
-- Copyright (C) 2011 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Avoision/Common.lua")
require("Effects/Ghost.lua")
require("Effects/ScriptedEntity.lua")

Display = { }

local defaultColor = Color.new(1, 1, 1)
local urgentColor = Color.new(1, 0, 0)
local scoreGhostPeriod = 300
local scoreGhostScaleMax = 3
local pointsGhostPeriod = 150
local pointsGhostScaleMax = 2
local bigScorePeriod = 1000
local bigScoreScaleMax = 2

function Display.new(board)
    local display = Entity.new()

    display.suppressEffects = false

    display.elements:add(Element.Text.new("Score: ", -200, 200, 0, Common.textWidth, Common.textHeight))
    display.score = display:addChild(NumberLabel.new(board, "score", -200 + (7 * Common.textWidth), 200, 0, Common.textWidth, Common.textHeight, "left", function(self, score)
        if not display.suppressEffects then
            Ghost.new(self, scoreGhostPeriod, scoreGhostScaleMax, true, nil, -#self.textElement.text * Common.textWidth / 2 * (scoreGhostScaleMax - 1), -Common.textHeight / 2 * (scoreGhostScaleMax - 1)):setParent(display)
        end
    end))

    display.elements:add(Element.Text.new("Points: ", 200 - (10 * Common.textWidth), 200, 0, Common.textWidth, Common.textHeight))
    display.points = display:addChild(NumberLabel.new(board, "points", 200, 200, 0, Common.textWidth, Common.textHeight, "right", function(self, points)
        if points <= 5 * board.difficulty then
            self.textElement.color = urgentColor
        else
            self.textElement.color = defaultColor
        end

        if not display.suppressEffects then
            Ghost.new(self, pointsGhostPeriod, pointsGhostScaleMax, true, nil, #self.textElement.text * Common.textWidth / 2 * (pointsGhostScaleMax - 1), -Common.textHeight / 2 * (pointsGhostScaleMax - 1)):setParent(display)
        end
    end))

    function display.reset(self)
        display.suppressEffects = true
        self.score:update()
        self.points:update()
        display.suppressEffects = false

        if isObject(self.bigScore) then
            self:removeChild(self.bigScore)
            self.bigScore = nil
        end
    end

    function display.emphasizeScore(self)
        local content = "Score: " .. self.score.textElement.text
        self.bigScore = ScriptedEntity.new({ Element.Image.new("Images/ScoreBackground.png", Common.textWidth * #content / 2, Common.textHeight / 2, 0, Common.textWidth * (#content + 1), Common.textHeight, 0, Color.new(1, 1, 1, 0.85)),
                                             Element.Text.new(content, 0, 0, 0.2, Common.textWidth, Common.textHeight) },
                                           { { 0, -200, 200, 0.1, 1, 1, 0, Color.new(1, 1, 1, 0.25) },
                                             { bigScorePeriod, -#content * Common.textWidth * bigScoreScaleMax / 2, -Common.textHeight * bigScoreScaleMax / 2 + 50, 0.1, bigScoreScaleMax, bigScoreScaleMax, 0, Color.new(1, 1, 1) } })
        self:addChild(self.bigScore)
    end

    return display
end

