-- This file is a part of Avoision.
-- 
-- Copyright (C) 2012 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/UI.lua")
require("Avoision/Common.lua")
require("Menus/HighScore.lua")

HighScoreScreen = { }

function HighScoreScreen.new(firstLevel, levels)
    local highScoreForm = Form.newGrid(1, -200, 200, 400)

    highScoreForm.firstLevel = firstLevel
    highScoreForm.levels = levels

    -- Activate/cancel exit
    local title = Title.new("High Scores")

    title.getActive = nil

    function title.inputReceived(self, event, pressed)
        if pressed then
            if event == "activate" then
                Layers.pop()
            end
        end
    end

    -- Create form
    highScoreForm:add(title)
    highScoreForm:add(Separator.new())

    -- Add all entries
    local i = firstLevel
    local l = 0

    highScoreForm.difficultyToNameLabels = { }
    highScoreForm.difficultyToScoreLabels = { }

    while l < levels do
        local difficultyName = Common.difficultyLevelNames[i]

        if not isNil(difficultyName) then
            local list = highScores[i]
            local j = 1

            if l > 0 then
                highScoreForm:add(Separator.new())
            end

            highScoreForm:add(Label.new(difficultyName))
            highScoreForm.difficultyToNameLabels[i] = { }
            highScoreForm.difficultyToScoreLabels[i] = { }

            list:forEach(function(highScore)
                local nameLabel = Label.new("")
                local scoreLabel = Label.new("", "right")

                highScoreForm:add(Form.newNestedGrid(2, nameLabel, scoreLabel))
                highScoreForm.difficultyToNameLabels[i][j] = nameLabel
                highScoreForm.difficultyToScoreLabels[i][j] = scoreLabel
                j = j + 1
            end)

            i = i + 1
            l = l + 1
        else
            break
        end
    end

    function highScoreForm.refresh(self)
        -- Add all scores
        local i = self.firstLevel
        local l = 0

        while l < self.levels do
            if not isNil(Common.difficultyLevelNames[i]) then
                local list = highScores[i]
                local j = 1

                list:forEach(function(highScore)
                    highScoreForm.difficultyToNameLabels[i][j]:setText("  " .. highScore.name)
                    highScoreForm.difficultyToScoreLabels[i][j]:setText(highScore.score .. "  ")
                    j = j + 1
                end)

                i = i + 1
                l = l + 1
            else
                break
            end
        end
    end

    local highScoreScreen = FormLayer.new(highScoreForm)

    function highScoreScreen.formShown(self)
        highScoreForm:refresh()
    end

    return highScoreScreen
end

HighScoreMenu = { }

function HighScoreMenu.new()
    local screens = { }
    local i = Common.difficultyLevel.Easy
    local levelsPerScreen = 2

    while not isNil(Common.difficultyLevelNames[i]) do
        screens[#screens + 1] = HighScoreScreen.new(i, levelsPerScreen)
        i = i + levelsPerScreen
    end
    
    return { show = function()
        -- Push all screens
        local s = #screens

        while s >= 1 do
            screens[s]:show()
            s = s - 1
        end
    end }
end

