-- This file is a part of Avoision.
-- 
-- Copyright (C) 2010 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

ImageLabel = { }

local setXByAlignment = {
    left = function(self, x, y) self.x = x end,
    center = function(self, x, y) self.x = x + self:getSize() / 2 end,
    right = function(self, x, y) self.x = x + self:getSize() end,
}

local getOriginByAlignment = {
    left = function() return 0.5, -0.5 end,
    center = function() return 0, -0.5 end,
    right = function() return 1, -0.5 end,
}

function ImageLabel.new(image, desiredWidth, desiredHeight, alignment, stretch)
    local imageLabel = Entity.new()

    imageLabel.stretch = stretch
    imageLabel.width = desiredWidth
    imageLabel.height = desiredHeight
    imageLabel.totalWidth = 0
    imageLabel.totalHeight = 0
    imageLabel.imageElement = imageLabel.elements:add(Element.Image.new(image))

    -- Position image element to respect alignment
    if isString(alignment) then
        local getOrigin = getOriginByAlignment[alignment]

        if isFunction(getOrigin) then
            imageLabel.alignment = alignment
            imageLabel.imageElement.x, imageLabel.imageElement.y = getOrigin()
        end
    end

    function imageLabel.setLayer(self, layer)
        layer:addChild(self)
    end

    function imageLabel.getPosition(self)
        return self.x, self.y
    end

    function imageLabel.setPosition(self, x, y)
        local setXInternal = setXByAlignment[self.alignment]

        if not isFunction(setXInternal) then
            setXInternal = setXByAlignment.left
        end

        setXInternal(self, x, y)
        self.y = y
    end

    function imageLabel.getActive(self)
        return false
    end

    function imageLabel.getMinimumSize(self)
        return self.width, self.height
    end

    function imageLabel.getDesiredSize(self)
        return self:getMinimumSize()
    end

    function imageLabel.getSize(self)
        return self.totalWidth, self.totalHeight
    end

    function imageLabel.setSize(self, width, height)
        -- Only change size if stretching
        if isBoolean(self.stretch) and self.stretch then
            self.width = width
            self.height = height
        else
            if isString(self.alignment) and self.alignment == "right" then
                -- Add padding
                local extraPaddingX = width - self.width
                local extraPaddingY = height - self.height

                self.x = self.x + extraPaddingX
                self.y = self.y + extraPaddingY
            end

            self.totalWidth = width
            self.totalHeight = height
        end
    end

    return imageLabel
end


